/*
 * pka_functions.c
 *
 * API to use the large number arithmetic functions
 * supported by PKA engine.
 *
 */

#include "pka_functions.h"
#include <stdio.h>
#include <stdlib.h>


#define MULT_RAM_START 0
#define DIV_RAM_START 0
#define ADD_RAM_START 0
#define SUB_RAM_START 0
#define CMP_RAM_START 0
#define MODINV_RAM_START 0
#define MODEXP_RAM_START 0

bool pka_wait_poll(void);




/*
 * pka_mult():  large number multiplication.  Blocking function.
 *
 *	input:
 *		vectA   - pointer to buffer storing multiplicand
 *		vectAsz - size of multiplicand in 32-bit words
 *		vectB   - pointer to buffer storing multiplier
 *		vectBsz - size of multiplier in 32-bit words
 *		result  - pointer to buffer holding result (size of buffer should be at least vectAsz + vectBsz)
 *	output:
 *	    function returns the pointer to result buffer if successful and NULL on error
 */
uint32_t *pka_mult(uint32_t *vectA, uint32_t vectAsz, uint32_t *vectB, uint32_t vectBsz, uint32_t *result){

	volatile uint32_t *PKA_RAM;
	int i;

	// write inputs to PKA RAM
	PKA_RAM = (volatile uint32_t *)(pREG_PKA0_RAM);

	/************
	 * MULTIPLY
	 *
	 * Vector A - Multiplicand
	 * Vector B - Multiplier
	 * Vector C - Product
	 * Vector D - N/A
	 *
	 * Input restrictions: 0 < A_Len, B_Len >= Max_Len (128 x 32bit words = 4096bit)
	 *
	 * Memory allocation for result vector in PKA RAM: A_Len + B_Len + 6 (the 6 'scratchpad' words should be discarded)
	 *
	 * No overlap allowed for input
	 **************/

	// set up lengths
	if(vectAsz > 128 || vectBsz > 128){
		printf("vector size too big! A: %d, B: %d\n", vectAsz, vectBsz);
		return NULL;
	}

	*pREG_PKA0_ALEN = vectAsz;
	*pREG_PKA0_BLEN = vectBsz;

	// where the inputs and output will be in PKA RAM
	// The pointers are index to the 32-bit words, but the vectors must be 8-byte aligned, so
	// so the last bit in the pointers will always need to be 0.

	*pREG_PKA0_APTR = MULT_RAM_START;
	*pREG_PKA0_BPTR = MULT_RAM_START + ((vectAsz+1)&(0xfffffffe));  // set up B to be after A in the next 8-byte multiple
	*pREG_PKA0_CPTR = MULT_RAM_START + ((vectAsz+1)&(0xfffffffe)) + ((vectBsz+1)&(0xfffffffe));

	// copy input to PKA RAM
	for(i=0; i<vectAsz; i++)
		PKA_RAM[*pREG_PKA0_APTR + i] = vectA[i];

	for(i=0; i<vectBsz; i++)
		PKA_RAM[*pREG_PKA0_BPTR + i] = vectB[i];

	// start operation
	*pREG_PKA0_FUNC = BITM_PKA_FUNC_MULT | BITM_PKA_FUNC_RUN;


	if(pka_wait_poll() == true){
		for(i = 0; i < (vectAsz + vectBsz); i++)
			result[i] = PKA_RAM[*pREG_PKA0_CPTR + i];
		return result;
	}
	else
		return NULL;
}




/*
 * pka_div():  large number division.  Blocking function.
 *
 *	input:
 *		vectA   - pointer to buffer storing dividend
 *		vectAsz - size of multiplicand in 32-bit words
 *		vectB   - pointer to buffer storing divisor
 *		vectBsz - size of multiplier in 32-bit words
 *		result  - pointer to buffer holding result (if NULL, result will not be copied)
 *		remainder - pointer to the buffer holding remainder (if NULL, result will not be copied)
 *	output:
 *	    function returns the pointer to result buffer if successful and NULL on error
 *
 *	restriction:  Most significand word of vector B cannot be 0
 */
uint32_t *pka_div(uint32_t *vectA, uint32_t vectAsz, uint32_t *vectB, uint32_t vectBsz, uint32_t *result, uint32_t *remainder){

	volatile uint32_t *PKA_RAM;
	int i;

	// write inputs to PKA RAM
	PKA_RAM = (volatile uint32_t *)(pREG_PKA0_RAM);

	// set up lengths
	if(vectAsz > 128 || vectBsz > 128 || vectAsz < 1 || vectBsz < 1 || vectAsz < vectBsz){
		printf("vector size bad! A: %d, B: %d\n", vectAsz, vectBsz);
		return NULL;
	}

	*pREG_PKA0_ALEN = vectAsz;
	*pREG_PKA0_BLEN = vectBsz;

	// where the inputs and output will be in PKA RAM
	// The pointers are index to the 32-bit words, but the vectors must be 8-byte aligned, so
	// so the last bit in the pointers will always need to be 0.

	*pREG_PKA0_APTR = DIV_RAM_START;
	*pREG_PKA0_BPTR = DIV_RAM_START + ((vectAsz+1)&(0xfffffffe));  // set up B to be after A in the next 8-byte multiple
	*pREG_PKA0_CPTR = DIV_RAM_START + ((vectAsz+1)&(0xfffffffe)) + ((vectBsz+1)&(0xfffffffe));
	*pREG_PKA0_DPTR = DIV_RAM_START + 2*((vectAsz+1)&(0xfffffffe)) + ((vectBsz+1)&(0xfffffffe));

	// copy input to PKA RAM
	for(i=0; i<vectAsz; i++)
		PKA_RAM[*pREG_PKA0_APTR + i] = vectA[i];

	for(i=0; i<vectBsz; i++)
		PKA_RAM[*pREG_PKA0_BPTR + i] = vectB[i];

	// start operation
	*pREG_PKA0_FUNC = BITM_PKA_FUNC_DIV | BITM_PKA_FUNC_RUN;


	if(pka_wait_poll() == true){
		if(result != NULL){
			for(i = 0; i < (vectAsz - vectBsz); i++)     // copy quotient to result buffer
				result[i] = PKA_RAM[*pREG_PKA0_DPTR + i];

			if(PKA_RAM[*pREG_PKA0_DPTR + i] != 0)
				printf("!MSW not 0 from division!\n");   // result size should really be A_Len-B_Len+1 but we'll extend past the buffer
		}
		if(remainder != NULL)
			for(i = 0; i < vectBsz; i++)
				remainder[i] = PKA_RAM[*pREG_PKA0_CPTR + i];

		return result;
	}
	else
		return NULL;
}







/*
 * pka_add():  large number addition.  Blocking function.
 *
 *	input:
 *		vectA   - pointer to buffer storing addend
 *		vectAsz - size of multiplicand in 32-bit words
 *		vectB   - pointer to buffer storing addend
 *		vectBsz - size of multiplier in 32-bit words
 *		result  - pointer to buffer holding sum (size of buffer should be at least MAX(vectAsz, vectBsz)+1 )
 *	output:
 *	    function returns the pointer to result buffer if successful and NULL on error
 */
uint32_t *pka_add(uint32_t *vectA, uint32_t vectAsz, uint32_t *vectB, uint32_t vectBsz, uint32_t *result){
	int i;
	volatile uint32_t *PKA_RAM;

	// write inputs to PKA RAM
	PKA_RAM = (volatile uint32_t *)(pREG_PKA0_RAM);

	// set up lengths
	if(vectAsz > 128 || vectBsz > 128){
		printf("vector size too big! A: %d, B: %d\n", vectAsz, vectBsz);
		return NULL;
	}

	*pREG_PKA0_ALEN = vectAsz;
	*pREG_PKA0_BLEN = vectBsz;

	// setting up word offsets of input and results vectors in memory
	*pREG_PKA0_APTR = ADD_RAM_START;
	*pREG_PKA0_BPTR = ADD_RAM_START +  ((vectAsz+1)&(0xfffffffe));  // set up B to be after A in the next 8-byte multiple
	*pREG_PKA0_CPTR = ADD_RAM_START +  ((vectAsz+1)&(0xfffffffe)) + ((vectBsz+1)&(0xfffffffe));


	// addend 1
	for(i=0; i<vectAsz; i++)
		PKA_RAM[*pREG_PKA0_APTR + i] = vectA[i];

	// addend 2
	for(i=0; i<vectBsz; i++)
		PKA_RAM[*pREG_PKA0_BPTR + i] = vectB[i];

	// start operation
	*pREG_PKA0_FUNC = BITM_PKA_FUNC_ADD | BITM_PKA_FUNC_RUN;

	if(pka_wait_poll() == true){
		// move sum to output buffer.  we assume that larger size buffer
		// is padded enough msbs with 0's so we don't have to worry about a carry
		// bit overflowing to another word.
		for(i = 0; i < (max(vectAsz, vectBsz)); i++)
			result[i] = PKA_RAM[*pREG_PKA0_CPTR + i];
		return result;
	}
	else
		return NULL;

}

/*
 * pka_sub():  large number subtraction.  Blocking function.
 *
 *	input:
 *		vectA   - pointer to buffer storing minuend
 *		vectAsz - size of multiplicand in 32-bit words
 *		vectB   - pointer to buffer storing subtrahend
 *		vectBsz - size of multiplier in 32-bit words
 *		result  - pointer to buffer holding difference
 *	output:
 *	    function returns the pointer to result buffer if successful and NULL on error
 */
uint32_t *pka_sub(uint32_t *vectA, uint32_t vectAsz, uint32_t *vectB, uint32_t vectBsz, uint32_t *result){
	int i;
	volatile uint32_t *PKA_RAM;

	// write inputs to PKA RAM
	PKA_RAM = (volatile uint32_t *)(pREG_PKA0_RAM);

	// set up lengths
	if(vectAsz > 128 || vectBsz > 128){
		printf("vector size too big! A: %d, B: %d\n", vectAsz, vectBsz);
		return NULL;
	}

	*pREG_PKA0_ALEN = vectAsz;
	*pREG_PKA0_BLEN = vectBsz;

	// setting up word offsets of input and results vectors in memory
	*pREG_PKA0_APTR = SUB_RAM_START;
	*pREG_PKA0_BPTR = SUB_RAM_START +  ((vectAsz+1)&(0xfffffffe));  // set up B to be after A in the next 8-byte multiple
	*pREG_PKA0_CPTR = SUB_RAM_START +  ((vectAsz+1)&(0xfffffffe)) + ((vectBsz+1)&(0xfffffffe));


	// addend 1
	for(i=0; i<vectAsz; i++)
		PKA_RAM[*pREG_PKA0_APTR + i] = vectA[i];

	// addend 2
	for(i=0; i<vectBsz; i++)
		PKA_RAM[*pREG_PKA0_BPTR + i] = vectB[i];

	// start operation
	*pREG_PKA0_FUNC = BITM_PKA_FUNC_SUB | BITM_PKA_FUNC_RUN;

	if(pka_wait_poll() == true){
		// move difference to output buffer.  we assume that larger size buffer
		// is padded enough msbs with 0's so we don't have to worry about a carry
		// bit overflowing to another word.
		for(i = 0; i < (max(vectAsz, vectBsz)); i++)
			result[i] = PKA_RAM[*pREG_PKA0_CPTR + i];
		return result;
	}
	else
		return NULL;

}



/*
 * pka_cmp():  large number comparison.  Blocking function.
 *
 *	input:
 *		vectA   - pointer to buffer storing minuend
 *		vectB   - pointer to buffer storing subtrahend
 *		vectSz - size of multiplier in 32-bit words
 *	output:
 *	    0 if A = B
 *	    1 if A > B
 *	   -1 if A < B
 *	   -2 on error
 */
int32_t pka_cmp(uint32_t *vectA, uint32_t *vectB, uint32_t vectSz){
	int i;
	uint32_t t;
	volatile uint32_t *PKA_RAM;

	// write inputs to PKA RAM
	PKA_RAM = (volatile uint32_t *)(pREG_PKA0_RAM);

	// set up lengths
	if(vectSz > 128){
		printf("vector size too big! A: %d\n", vectSz);
		return (int32_t)(NULL);
	}

	*pREG_PKA0_ALEN = vectSz;  // engine will only use length_A for both numbers

	// setting up word offsets of input and results vectors in memory
	*pREG_PKA0_APTR = CMP_RAM_START;
	*pREG_PKA0_BPTR = CMP_RAM_START +  ((vectSz+1)&(0xfffffffe));  // set up B to be after A in the next 8-byte multiple


	for(i=0; i<vectSz; i++){
		PKA_RAM[*pREG_PKA0_APTR + i] = vectA[i];
		PKA_RAM[*pREG_PKA0_BPTR + i] = vectB[i];
	}

	// start operation
	*pREG_PKA0_FUNC = BITM_PKA_FUNC_CMP | BITM_PKA_FUNC_RUN;

	if(pka_wait_poll() == true){
		t = *pREG_PKA0_COMPARE;
		if(t & BITM_PKA_COMPARE_AEQB)
			return 0;
		else if(t & BITM_PKA_COMPARE_AGTB)
			return 1;
		else if(t & BITM_PKA_COMPARE_ALTB)
			return -1;
		else
			return -2;
	}
	else
		return -2;


}


/*
 *
 * pka_modinv(): Compute modular inverse of large number.  Blocking function.
 * A ^-1 mod B -> D
 *
 *	input:
 *		vectA   - pointer to buffer storing number to find inverse of
 *		vectAsz - size of number in 32-bit words
 *		vectB   - pointer to buffer storing modulus
 *		vectBsz - size of modulus in 32-bit words
 *	    result  - pointer to buffer storing result
 *
 * Input restrictions:
 * 1. 0 < A_Len <= Max_Len
 * 2. 0 < B_Len <= Max_Len
 * 3. Modulus in B must be ODD
 * 4. Modulus in B must not have a value of '1' (result is undefined with not error indicated)
 * 5. The highest word of the modulus vector, as indicated by B_Len, may not be 0
 *
 * Overlap:
 * Scratchpad area starting at D may not overlap with any of the other vectors
 *
 * Also, during execution of a ModInv operation, the last 34 bytes of the PKA RAM are used as general
 * scratchpad for the Sequencers program execution. This (fixed location) area may not overlap with any of the
 * input vectors and/or the D vector scratchpad area during execution.
 *
 * Return code in PKA_SHIFT reg:
 *  0 -> success; vector D hold result
 *  7 -> no inverse exists (GCD(A,B) != 1, i.e. A and B have common factors); result undefined
 *  31 -> error.  Modulus EVEN; result undefined
 *
 */


uint32_t pka_modinv(uint32_t *vectA, uint32_t vectAsz, uint32_t *vectB, uint32_t vectBsz, uint32_t *result){

	int i;
	uint32_t t;
	volatile uint32_t *PKA_RAM;

	// write inputs to PKA RAM
	PKA_RAM = (volatile uint32_t *)(pREG_PKA0_RAM);

	// set up lengths
	*pREG_PKA0_ALEN = vectAsz;
	*pREG_PKA0_BLEN = vectBsz;

	// setting up word offsets of input and results vectors in memory
	*pREG_PKA0_APTR = SUB_RAM_START;
	*pREG_PKA0_BPTR = SUB_RAM_START +  ((vectAsz+1)&(0xfffffffe));  // set up B to be after A in the next 8-byte multiple
	*pREG_PKA0_DPTR = SUB_RAM_START +  ((vectAsz+1)&(0xfffffffe)) + ((vectBsz+1)&(0xfffffffe));


	// Vector A: number to invert
	for(i=0; i<vectAsz; i++)
		PKA_RAM[*pREG_PKA0_APTR + i] = vectA[i];

	// Vector B: modulus
	for(i=0; i<vectBsz; i++)
		PKA_RAM[*pREG_PKA0_BPTR + i] = vectB[i];


	// start operation
	*pREG_PKA0_FUNC = (7<<12) | BITM_PKA_FUNC_RUN;  // ModInv, run


	if(pka_wait_poll() == true  && *pREG_PKA0_SHIFT == 0 ){
		if(result != NULL)
			for(i=0; i<vectBsz; i++)
				result[i] = PKA_RAM[*pREG_PKA0_DPTR + i];
		return 0;
	}

	return *pREG_PKA0_SHIFT;

}




/************
 * pka_modexp(): Modular exponentiation.  Blocking function.
 * C^A mod B -> D
 *
 *	input:
 *		exp  (A)- pointer to exponent
 *		mod  (B)- pointer to modulus
 *		base (C)- pointer to base
 *		szWds - size of vectors in 32-bit words.  Exponent, modulus and base must all be same size.
 *	output:
 *		result (D) - pointer to output buffer
 *
 * Input restrictions:
 * 	1. 0 < A_Len <= Max_Len
 * 	2. 1 < B_len <= Max_Len
 * 	3. Modulus B must be odd
 * 	4. Modulus B > 2^32
 * 	5. Base C < Modulus B
 * 	6. Vectors B and C must be followed by an empty 32-bit 'buffer' word
 *
 * **************/

uint32_t *pka_modexp(uint32_t *exp, uint32_t expSzWds, uint32_t *mod, uint32_t *base, uint32_t ModBaseSzWds, uint32_t *result){

	int i;
	volatile uint32_t *PKA_RAM;

	// write inputs to PKA RAM
	PKA_RAM = (volatile uint32_t *)(pREG_PKA0_RAM);

	while(expSzWds>1 && exp[expSzWds-1]==0)
		expSzWds--;

	// set up lengths
	*pREG_PKA0_ALEN = expSzWds;      // size of exponent
	*pREG_PKA0_BLEN = ModBaseSzWds;  // size of modulo and base


	// Setting up word offsets of input and results vectors in memory
	// For mod exp functions vectors B and C must be followed by an empty 32-bit buffer word, hence the extra +2
	*pREG_PKA0_APTR = MODEXP_RAM_START;
	*pREG_PKA0_BPTR = MODEXP_RAM_START + ((expSzWds+1)&(0xfffffffe));
	*pREG_PKA0_CPTR = MODEXP_RAM_START + ((expSzWds+1)&(0xfffffffe)) + ((ModBaseSzWds+1)&(0xfffffffe)) + 2;
	*pREG_PKA0_DPTR = MODEXP_RAM_START + ((expSzWds+1)&(0xfffffffe)) + 2*((ModBaseSzWds+1)&(0xfffffffe)) + 2;

	// Vector A: exponent
	for(i=0; i<expSzWds; i++)
		PKA_RAM[*pREG_PKA0_APTR + i] = exp[i];

	// Vector B: modulus
	for(i=0; i<ModBaseSzWds; i++)
		PKA_RAM[*pREG_PKA0_BPTR + i] = mod[i];

	if((mod[0] & 1) == 0){
		printf("pka_modexp() Error: Modulo is even\n");
		return NULL;
	}


	// Vector C: base
	for(i=0; i<ModBaseSzWds; i++)
		PKA_RAM[*pREG_PKA0_CPTR + i] = base[i];


	// start operation
	//*pREG_PKA0_FUNC = (4<<12) | BITM_PKA_FUNC_RUN;  // ExpMod-Act2, run
	*pREG_PKA0_FUNC = (2<<12) | BITM_PKA_FUNC_RUN;  // ExpMod-Act4, run

//	*pREG_PKA0_FUNC  = (6<<12) | BITM_PKA_FUNC_RUN;   // ExpMod-var, run
//	*pREG_PKA0_SHIFT = 16;                            // indicate how many "odd powers" to use.  Use with ExpMod-var

	if(pka_wait_poll() == true){
		for(i=0; i<ModBaseSzWds; i++)
			result[i] = PKA_RAM[*pREG_PKA0_DPTR + i];
		return result;
	}
	else
		return NULL;
}


/************
 * pka_modexpcrt(): Modular exponentiation using Chinese Remainder Theorem.  Blocking function.
 * C^A mod B -> D
 *
 *	input:
 *		dp   (A)- pointer to dp (= d mod(p-1))
 *		dq   (A)- pointer to dq (= d mod(q-1))
 *		p    (B)- pointer to p
 *		q    (B)- pointer to q
 *		qinv (C)- pointer to q inverse
 *		base (D)- pointer to input
 *		szWds - size of vectors in 32-bit words.  Exponent, modulus and base must all be same size.
 *	output:
 *		result (D) - pointer to output buffer
 *		           - computes:
 *		                     m1 = base^dp mod p
 *		                     m2 = base^dq mod q
 *		                     h  = ((m1 - m2) * qinv) mod p
 *		                     result = m2 + q * h
 *
 *
 * Input restrictions:
 * 	1. 0 < A_Len <= Max_Len
 * 	2. 1 < B_len <= Max_Len
 * 	3. Mod P and Mod Q must be odd
 * 	4. Mod P > Mod Q > 2^32
 * 	5. Mod P and Mod Q must be co-prime (GCD=1)
 * 	6. 0 < dp < (Mod P-1)
 * 	7. 0 < dq < (Mod Q-1)
 * 	8. qinv * q = 1 MOD P
 * 	9. base < (Mod P * Mod Q)  (i.e. < n)
 * 	10. Mod P and Mod Q must be followed by an empty "buffer" word
 *
 * **************/

uint32_t *pka_modexpcrt(uint32_t *dp, uint32_t *dq, uint32_t *p, uint32_t *q, uint32_t *qinv, uint32_t *base, uint32_t SzWds, uint32_t *result){

	int i;
	volatile uint32_t *PKA_RAM;
	uint32_t expLen = (SzWds+1)/2, primeFactorsLen = (SzWds+1)/2;
	uint32_t szDP = expLen, szDQ = expLen, szP = primeFactorsLen, szQ = primeFactorsLen;


	// write inputs to PKA RAM
	PKA_RAM = (volatile uint32_t *)(pREG_PKA0_RAM);

	while(szDP>0 && dp[szDP-1] == 0)
		szDP --;

	while(szDQ>0 && dq[szDQ-1] == 0)
		szDQ --;

	while(szP>1 && p[szP-1] == 0)
		szP --;

	while(szQ>1 && q[szQ-1] == 0)
		szQ --;

	if(szP <= 1 || szQ <= 1){
		printf("pka_modexpcrt() Error: one of the prime factors is not big enough\n");
		return NULL;
	}

	// set up lengths
	*pREG_PKA0_ALEN = expLen;             // size of exponents, dp, dq
	*pREG_PKA0_BLEN = primeFactorsLen;    // size of prime factors, p, q and q^-1



	// Setting up word offsets of input and results vectors in memory
	// For mod exp functions vectors B and C must be followed by an empty 32-bit buffer word, hence the extra +2
	*pREG_PKA0_APTR = MODEXP_RAM_START;
	*pREG_PKA0_BPTR = *pREG_PKA0_APTR + 2*((expLen+1)&(0xfffffffe));
	*pREG_PKA0_CPTR = *pREG_PKA0_BPTR + 2*((primeFactorsLen+2)&(0xfffffffe));
	*pREG_PKA0_DPTR = *pREG_PKA0_CPTR +   ((primeFactorsLen+1)&(0xfffffffe));

	// Vector A: exponents
	for(i=0; i<expLen; i++)
		PKA_RAM[*pREG_PKA0_APTR + i] = dp[i];

	for(i=((expLen+1)&(0xfffffffe)); i< (((expLen+1)&(0xfffffffe)) + expLen); i++)
		PKA_RAM[*pREG_PKA0_APTR + i] = dq[i-((expLen+1)&(0xfffffffe))];

	// Vector B: prime factors
	for(i=0; i<primeFactorsLen; i++)
		PKA_RAM[*pREG_PKA0_BPTR + i] =p[i];
	PKA_RAM[*pREG_PKA0_BPTR + i] = 0;                              //buffer word

	for(i=((primeFactorsLen+2)&(0xfffffffe)); i<(((primeFactorsLen+2)&(0xfffffffe))+primeFactorsLen); i++)
		PKA_RAM[*pREG_PKA0_BPTR + i] =q[i-((primeFactorsLen+2)&(0xfffffffe))];
	PKA_RAM[*pREG_PKA0_BPTR + i] = 0;                              //buffer word

	// Vector C: Q inverse
	for(i=0; i<primeFactorsLen; i++)
		PKA_RAM[*pREG_PKA0_CPTR + i] =qinv[i];

	// Vector D: base (input)
	for(i=0; i<2*primeFactorsLen; i++){
		if(i < SzWds)
			PKA_RAM[*pREG_PKA0_DPTR + i] =base[i];
		else
			PKA_RAM[*pREG_PKA0_DPTR + i] =0;
	}

	if((p[0] & 1) == 0 || (q[0] & 1) == 0 ){
		printf("pka_modexpcrt() Error: one of the modulos is even\n");
		return NULL;
	}

	// start operation
	*pREG_PKA0_SHIFT = 16;                            // indicate how many "odd powers" to use.  Use with ExpMod-var
	*pREG_PKA0_FUNC = (1<<12) | BITM_PKA_FUNC_RUN;  // ExpMod-CRT, run

	if(pka_wait_poll() == true){
		for(i=0; i<2*primeFactorsLen; i++)
			//result[i] = PKA_RAM[*pREG_PKA0_DPTR + 2*primeFactorsLen + i];
			result[i] = PKA_RAM[*pREG_PKA0_DPTR + i];
		return result;
	}
	else
		return NULL;
}


/*
 * pka_wait_poll()
 *
 * Helper function.  Polls on status bit until done operation completes.
 *
 */
bool pka_wait_poll(){
	uint32_t t, i, counter=0;

#define MAX_COUNTER_WAIT 100

	t = *pREG_PKA0_FUNC;
	while (t & BITM_PKA_FUNC_RUN ){
		t = *pREG_PKA0_FUNC;
		for(i=0; i<100; i++)        // delay loop
			;

		if(counter++ > MAX_COUNTER_WAIT)
			return false;           //
	}

	return true;
}
